"""REST client handling, including WorkatoStream base class."""

from pathlib import Path
from typing import Any, Dict, Iterable, Optional

import requests
from singer_sdk.helpers.jsonpath import extract_jsonpath
from singer_sdk.streams import RESTStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class WorkatoStream(RESTStream):
    """Workato stream class."""

    url_base = "https://www.workato.com"

    records_jsonpath = "$[*]"
    current_page = None

    @property
    def http_headers(self) -> dict:
        """Return the http headers needed."""
        return {
            "x-user-token": self.config.get("user_token"),
            "x-user-email": self.config.get("user_email"),
        }

    def get_next_page_token(
        self, response: requests.Response, previous_token: Optional[Any]
    ) -> Optional[Any]:
        """Return a token for identifying next page or None if no more pages."""
        next_page_token: Optional[int] = None
        if self.current_page:
            all_matches = extract_jsonpath(self.records_jsonpath, response.json())
            records_cnt = sum([1 for m in all_matches])
            next_page_token = self.current_page + 1 if records_cnt == 100 else None

        return next_page_token

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization."""
        params: dict = {"per_page": 100}
        if next_page_token:
            params["page"] = next_page_token
            self.current_page = next_page_token
        if self.replication_key:
            params["order"] = "default"
        return params

    def parse_response(self, response: requests.Response) -> Iterable[dict]:
        """Parse the response and return an iterator of result rows."""
        yield from extract_jsonpath(self.records_jsonpath, input=response.json())

    def post_process(self, row: dict, context: Optional[dict] = None) -> Optional[dict]:
        """As needed, append or transform raw data to match expected structure."""
        return row



"""Stream type classes for tap-workato."""

from typing import Optional

from singer_sdk import typing as th

from tap_workato.client import WorkatoStream


class ApiCollectionsStream(WorkatoStream):
    """Stream for extracting API Collections."""

    name = "api_collections"
    path = "/api/api_collections"
    primary_keys = ["id"]
    replication_key = None
    current_page = 1
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("version", th.StringType),
        th.Property("url", th.StringType),
        th.Property("api_spec_url", th.StringType),
    ).to_dict()


class ApiClientsStream(WorkatoStream):
    """Stream for extracting API Clients."""

    name = "api_clients"
    path = "/api/api_clients"
    primary_keys = ["id"]
    replication_key = None
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()


class ConnectionsStream(WorkatoStream):
    """Stream for extracting Connections."""

    name = "connections"
    path = "/api/connections"
    primary_keys = ["id"]
    replication_key = None
    schema = th.PropertiesList(
        th.Property("application", th.StringType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("description", th.StringType),
        th.Property("authorized_at", th.DateTimeType),
        th.Property("authorization_status", th.StringType),
        th.Property("authorization_error", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("external_id", th.IntegerType),
        th.Property("folder_id", th.IntegerType),
        th.Property("parent_id", th.IntegerType),
    ).to_dict()


class FoldersStream(WorkatoStream):
    """Stream for extracting Folders."""

    name = "folders"
    path = "/api/folders"
    primary_keys = ["id"]
    replication_key = None
    current_page = 1
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("parent_id", th.IntegerType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()


class RecipesStream(WorkatoStream):
    """Stream for extracting recipes."""

    name = "recipes"
    path = "/api/recipes"
    primary_keys = ["id"]
    replication_key = None
    records_jsonpath = "$.items[*]"
    current_page = 1
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("user_id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("copy_count", th.IntegerType),
        th.Property("trigger_application", th.StringType),
        th.Property("action_applications", th.ArrayType(th.StringType)),
        th.Property("applications", th.ArrayType(th.StringType)),
        th.Property("description", th.StringType),
        th.Property("parameters_schema", th.ArrayType(th.StringType)),
        th.Property("parameters", th.ObjectType()),
        th.Property("folder_id", th.IntegerType),
        th.Property("running", th.BooleanType),
        th.Property("job_succeeded_count", th.IntegerType),
        th.Property("job_failed_count", th.IntegerType),
        th.Property("lifetime_task_count", th.IntegerType),
        th.Property("last_run_at", th.DateTimeType),
        th.Property("stopped_at", th.DateTimeType),
        th.Property("version_no", th.IntegerType),
        th.Property("webhook_url", th.StringType),
        th.Property("stop_cause", th.StringType),
        th.Property("code", th.StringType),
        th.Property(
            "config",
            th.ArrayType(
                th.ObjectType(
                    th.Property("keyword", th.StringType),
                    th.Property("name", th.StringType),
                    th.Property("provider", th.StringType),
                    th.Property("skip_validation", th.BooleanType),
                    th.Property("account_id", th.IntegerType),
                )
            ),
        ),
    ).to_dict()

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {"recipe_id": record["id"]}


class JobsStream(WorkatoStream):
    """Stream for extracting Jobs."""

    name = "jobs"
    path = "/api/recipes/{recipe_id}/jobs"
    primary_keys = ["recipe_id"]
    replication_key = None
    parent_stream_type = RecipesStream
    schema = th.PropertiesList(
        th.Property("recipe_id", th.IntegerType),
        th.Property("job_succeeded_count", th.IntegerType),
        th.Property("job_failed_count", th.IntegerType),
        th.Property("job_count", th.IntegerType),
        th.Property(
            "items",
            th.ArrayType(
                th.ObjectType(
                    th.Property("id", th.IntegerType),
                    th.Property("flow_run_id", th.IntegerType),
                    th.Property("completed_at", th.DateTimeType),
                    th.Property("started_at", th.DateTimeType),
                    th.Property("title", th.StringType),
                    th.Property("is_poll_error", th.BooleanType),
                    th.Property("error", th.StringType),
                    th.Property("is_error", th.BooleanType),
                    th.Property(
                        "error_parts",
                        th.ObjectType(
                            th.Property("adapter", th.StringType),
                            th.Property("error_type", th.StringType),
                            th.Property("message", th.StringType),
                            th.Property("error_id", th.StringType),
                            th.Property("error_at", th.DateTimeType),
                            th.Property("input", th.StringType),
                            th.Property("inner_message", th.StringType),
                        ),
                    ),
                )
            ),
        ),
    ).to_dict()

    def post_process(  # type: ignore[override]
        self, row: dict, context: dict
    ) -> Optional[dict]:
        """As needed, append or transform raw data to match expected structure."""
        row["recipe_id"] = context["recipe_id"]
        return row


class OnPremGroupsStream(WorkatoStream):
    """Stream for extracting On-prem Groups."""

    name = "on_prem_groups"
    path = "/api/on_prem_groups"
    primary_keys = ["id"]
    replication_key = None
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
    ).to_dict()


class OnPremAgentsStream(WorkatoStream):
    """Stream for extracting On-prem Agents."""

    name = "on_prem_agents"
    path = "/api/on_prem_agents"
    primary_keys = ["id"]
    replication_key = None
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("on_prem_group_id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("cn", th.StringType),
        th.Property("os", th.StringType),
        th.Property("awaiting_setup", th.BooleanType),
        th.Property("enabled", th.BooleanType),
    ).to_dict()


class RolesStream(WorkatoStream):
    """Stream for extracting Custom Roles."""

    name = "roles"
    path = "/api/roles"
    primary_keys = ["id"]
    replication_key = None
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("inheritable", th.BooleanType),
        th.Property("folder_ids", th.ArrayType(th.IntegerType)),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()


# Streams For Managed Users / Customer Account
class CustomerAccountsStream(WorkatoStream):
    """Stream for extracting Managed Users or Customer Accounts."""

    name = "customer_accounts"
    path = "/api/managed_users"
    primary_keys = ["id"]
    replication_key = None
    records_jsonpath = "$.result[*]"
    current_page = 1
    schema = th.PropertiesList(
        th.Property("id", th.IntegerType),
        th.Property("external_id", th.StringType),
        th.Property("name", th.StringType),
        th.Property("notification_email", th.StringType),
        th.Property("error_notification_emails", th.StringType),
        th.Property("admin_notification_emails", th.StringType),
        th.Property("whitelisted_apps", th.ArrayType(th.StringType)),
        th.Property("plan_id", th.StringType),
        th.Property("origin_url", th.StringType),
        th.Property("frame_ancestors", th.StringType),
        th.Property("trial", th.BooleanType),
        th.Property("in_trial", th.BooleanType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("current_billing_period_start", th.DateTimeType),
        th.Property("current_billing_period_end", th.DateTimeType),
        th.Property("task_limit_adjustment", th.StringType),
        th.Property("task_limit", th.IntegerType),
        th.Property("task_count", th.IntegerType),
        th.Property("active_connection_limit", th.IntegerType),
        th.Property("active_connection_count", th.IntegerType),
        th.Property("active_recipe_count", th.IntegerType),
    ).to_dict()

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {"customer_account_id": record["id"]}


class CustomerChildStreams(WorkatoStream):
    """Parent Stream for all children to the CustomerAccountsStream for DRY code."""

    parent_stream_type = CustomerAccountsStream
    primary_keys = ["customer_account_id", "id"]
    replication_key = None

    def post_process(  # type: ignore[override]
        self, row: dict, context: dict
    ) -> Optional[dict]:
        """As needed, append or transform raw data to match expected structure."""
        row["customer_account_id"] = context["customer_account_id"]
        return row


class CustomerMembersStream(CustomerChildStreams):
    """Stream for extracting customers' connections."""

    name = "customer_members"
    path = "/api/managed_users/{customer_account_id}/members"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("grant_type", th.StringType),
        th.Property("role_name", th.StringType),
        th.Property("external_id", th.StringType),
        th.Property("name", th.StringType),
        th.Property("email", th.StringType),
        th.Property("time_zone", th.StringType),
    ).to_dict()


class CustomerConnectionsStream(CustomerChildStreams):
    """Stream for extracting customers' connections."""

    name = "customer_connections"
    path = "/api/managed_users/{customer_account_id}/connections"
    records_jsonpath = "$.result[*]"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("provider", th.StringType),
        th.Property("authorized_at", th.DateTimeType),
        th.Property("authorization_status", th.StringType),
        th.Property("authorization_error", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("external_id", th.StringType),
        th.Property("folder_id", th.IntegerType),
        th.Property("parent_id", th.IntegerType),
    ).to_dict()


class CustomerFoldersStream(CustomerChildStreams):
    """Stream for extracting customers' folders."""

    name = "customer_folders"
    path = "/api/managed_users/{customer_account_id}/folders"
    records_jsonpath = "$.result[*]"
    current_page = 1
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("parent_id", th.IntegerType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()


class CustomerRecipesStream(CustomerChildStreams):
    """Stream for extracting customers' recipes."""

    name = "customer_recipes"
    path = "/api/managed_users/{customer_account_id}/recipes"
    records_jsonpath = "$.result[*]"
    current_page = 1
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("user_id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("copy_count", th.IntegerType),
        th.Property("trigger_application", th.StringType),
        th.Property("action_applications", th.ArrayType(th.StringType)),
        th.Property("applications", th.ArrayType(th.StringType)),
        th.Property("description", th.StringType),
        th.Property("parameters_schema", th.ArrayType(th.StringType)),
        th.Property("parameters", th.ObjectType()),
        th.Property("folder_id", th.IntegerType),
        th.Property("running", th.BooleanType),
        th.Property("job_succeeded_count", th.IntegerType),
        th.Property("job_failed_count", th.IntegerType),
        th.Property("lifetime_task_count", th.IntegerType),
        th.Property("last_run_at", th.DateTimeType),
        th.Property("stopped_at", th.DateTimeType),
        th.Property("version_no", th.IntegerType),
        th.Property("webhook_url", th.StringType),
        th.Property("stop_cause", th.StringType),
        th.Property("code", th.StringType),
        th.Property(
            "config",
            th.ArrayType(
                th.ObjectType(
                    th.Property("keyword", th.StringType),
                    th.Property("name", th.StringType),
                    th.Property("provider", th.StringType),
                    th.Property("skip_validation", th.BooleanType),
                    th.Property("account_id", th.IntegerType),
                )
            ),
        ),
    ).to_dict()

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {
            "customer_account_id": record["customer_account_id"],
            "recipe_id": record["id"],
        }


# not currently possible to hit this endpoint for managed users' recipes
# class CustomerJobsStream(CustomerChildStreams):
#     """Stream for extracting Jobs."""
#
#     name = "customer_jobs"
#     path = "/api/recipes/{recipe_id}/jobs"
#     primary_keys = ["customer_account_id", "recipe_id"]
#     replication_key = None
#     parent_stream_type = CustomerRecipesStream
#     schema = th.PropertiesList(
#         th.Property("customer_account_id", th.IntegerType),
#         th.Property("recipe_id", th.IntegerType),
#         th.Property("job_succeeded_count", th.IntegerType),
#         th.Property("job_failed_count", th.IntegerType),
#         th.Property("job_count", th.IntegerType),
#         th.Property(
#             "items",
#             th.ArrayType(
#                 th.ObjectType(
#                     th.Property("id", th.IntegerType),
#                     th.Property("flow_run_id", th.IntegerType),
#                     th.Property("completed_at", th.DateTimeType),
#                     th.Property("started_at", th.DateTimeType),
#                     th.Property("title", th.StringType),
#                     th.Property("is_poll_error", th.BooleanType),
#                     th.Property("error", th.StringType),
#                     th.Property("is_error", th.BooleanType),
#                     th.Property(
#                         "error_parts",
#                         th.ObjectType(
#                             th.Property("adapter", th.StringType),
#                             th.Property("error_type", th.StringType),
#                             th.Property("message", th.StringType),
#                             th.Property("error_id", th.StringType),
#                             th.Property("error_at", th.DateTimeType),
#                             th.Property("input", th.StringType),
#                             th.Property("inner_message", th.StringType),
#                         ),
#                     ),
#                 )
#             ),
#         ),
#     ).to_dict()
#
#     def post_process(self, row: dict, context: dict) -> Optional[dict]:
#         """As needed, append or transform raw data to match expected structure."""
#         row["customer_account_id"] = context["customer_account_id"]
#         row["recipe_id"] = context["recipe_id"]
#         return row


class CustomerApiCollectionsStream(CustomerChildStreams):
    """Stream for extracting customers' folders."""

    name = "customer_api_collections"
    path = "/api/managed_users/{customer_account_id}/api_collections"
    records_jsonpath = "$.result[*]"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
        th.Property("version", th.StringType),
        th.Property("url", th.StringType),
        th.Property("api_spec_url", th.StringType),
    ).to_dict()


class CustomerApiEndpointsStream(CustomerChildStreams):
    """Stream for extracting customers' folders."""

    name = "customer_api_endpoints"
    path = "/api/managed_users/{customer_account_id}/api_endpoints"
    records_jsonpath = "$.result[*]"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("api_collection_id", th.IntegerType),
        th.Property("flow_id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("method", th.StringType),
        th.Property("url", th.StringType),
        th.Property("legacy_url", th.StringType),
        th.Property("base_path", th.StringType),
        th.Property("path", th.StringType),
        th.Property("active", th.BooleanType),
        th.Property("legacy", th.BooleanType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()


class CustomerApiClientsStream(CustomerChildStreams):
    """Stream for extracting customers' folders."""

    name = "customer_api_clients"
    path = "/api/managed_users/{customer_account_id}/api_clients"
    records_jsonpath = "$.result[*]"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()

    def get_child_context(self, record: dict, context: Optional[dict]) -> dict:
        """Return a context dictionary for child streams."""
        return {
            "customer_account_id": record["customer_account_id"],
            "api_client_id": record["id"],
        }


class CustomerApiAccessProfilesStream(WorkatoStream):
    """Stream for extracting customers' folders."""

    name = "customer_api_access_profiles"
    path = "/api/managed_users/{customer_account_id}/api_access_profiles"
    primary_keys = ["customer_account_id", "api_client_id", "id"]
    replication_key = None
    parent_stream_type = CustomerApiClientsStream
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("api_client_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("external_id", th.StringType),
        th.Property("name", th.StringType),
        th.Property("notification_email", th.StringType),
        th.Property("plan_id", th.StringType),
        th.Property("in_trial", th.BooleanType),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()

    def post_process(  # type: ignore[override]
        self, row: dict, context: dict
    ) -> Optional[dict]:
        """As needed, append or transform raw data to match expected structure."""
        row["customer_account_id"] = context["customer_account_id"]
        row["api_client_id"] = context["api_client_id"]
        return row


class CustomerRolesStream(CustomerChildStreams):
    """Stream for extracting customers' folders."""

    name = "customer_roles"
    path = "/api/managed_users/{customer_account_id}/roles"
    schema = th.PropertiesList(
        th.Property("customer_account_id", th.IntegerType),
        th.Property("id", th.IntegerType),
        th.Property("name", th.StringType),
        th.Property("inheritable", th.BooleanType),
        th.Property("folder_ids", th.ArrayType(th.IntegerType)),
        th.Property("created_at", th.DateTimeType),
        th.Property("updated_at", th.DateTimeType),
    ).to_dict()



"""Workato tap class."""

from typing import List

from singer_sdk import Stream, Tap
from singer_sdk import typing as th

from tap_workato.streams import (
    ApiClientsStream,
    ApiCollectionsStream,
    ConnectionsStream,
    CustomerAccountsStream,
    CustomerApiAccessProfilesStream,
    CustomerApiClientsStream,
    CustomerApiCollectionsStream,
    CustomerApiEndpointsStream,
    CustomerConnectionsStream,
    CustomerFoldersStream,
    CustomerMembersStream,
    CustomerRecipesStream,
    CustomerRolesStream,
    FoldersStream,
    JobsStream,
    OnPremAgentsStream,
    OnPremGroupsStream,
    RecipesStream,
    RolesStream,
)

STREAM_TYPES = [
    ApiClientsStream,
    ApiCollectionsStream,
    ConnectionsStream,
    CustomerAccountsStream,
    CustomerApiAccessProfilesStream,
    CustomerApiClientsStream,
    CustomerApiCollectionsStream,
    CustomerApiEndpointsStream,
    CustomerConnectionsStream,
    CustomerFoldersStream,
    CustomerMembersStream,
    CustomerRecipesStream,
    CustomerRolesStream,
    FoldersStream,
    JobsStream,
    OnPremAgentsStream,
    OnPremGroupsStream,
    RecipesStream,
    RolesStream,
]


class TapWorkato(Tap):
    """Workato tap class."""

    name = "tap-workato"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "user_token",
            th.StringType,
            required=True,
            description="The token to authenticate against the Workato API service",
        ),
        th.Property(
            "user_email",
            th.StringType,
            required=True,
            description="The email address of the user paired with the token.",
        ),
        # th.Property(
        #     "start_date", stopped_after in recipes?
        #     th.DateTimeType,
        #     description="The earliest record date to sync"
        # ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        return [stream_class(tap=self) for stream_class in STREAM_TYPES]



"""Tests standard tap features using the built-in SDK tests library."""

from singer_sdk.testing import get_standard_tap_tests

from tap_workato.tap import TapWorkato

SAMPLE_CONFIG = {
    "user_email": "test@workato.blah",
    "user_token": "abc-123",
}


def json_resp(extras: dict = None) -> dict:
    """Utility function returning a common response for mocked API calls.

    Args:
        extras: items to be added to the contents of the json response.

    Returns:
        A json object that mocks the results of an API call.
    """
    contents = {
        "items": [
            {
                "id": 7498,
                "name": "Netsuite production",
            },
            {
                "id": 7302,
                "name": "Automation HR",
            },
        ]
    }

    if extras:
        for k, v in extras.items():
            contents[k] = v
    return contents


endpoints = [
    "https://www.workato.com/api/api_clients?per_page=100",
    "https://www.workato.com/api/api_collections?per_page=100",
    "https://www.workato.com/api/connections?per_page=100",
    "https://www.workato.com/api/roles?per_page=100",
    "https://www.workato.com/api/deployments?per_page=100",
    "https://www.workato.com/api/folders?per_page=100",
    "https://www.workato.com/api/on_prem_agents?per_page=100",
    "https://www.workato.com/api/on_prem_groups?per_page=100",
    "https://www.workato.com/api/recipes?per_page=100",
    "https://www.workato.com/api/recipes/7498/jobs?per_page=100",
    "https://www.workato.com/api/recipes/7302/jobs?per_page=100",
    "https://www.workato.com/api/managed_users?per_page=100",
]


# Run standard built-in tap tests from the SDK:
def test_standard_tap_tests(requests_mock):
    """Run standard tap tests from the SDK."""
    for endpoint in endpoints:
        requests_mock.get(endpoint, json=json_resp())
    tests = get_standard_tap_tests(TapWorkato, config=SAMPLE_CONFIG)
    for test in tests:
        test()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_workato', 'tap_workato.tests']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.25.1,<3.0.0', 'singer-sdk>=0.13.1,<0.14.0']

entry_points = \
{'console_scripts': ['tap-workato = tap_workato.tap:TapWorkato.cli']}

setup_kwargs = {
    'name': 'tap-workato',
    'version': '0.0.2',
    'description': '`tap-workato` is a Singer tap for Workato, built with the Meltano SDK for Singer Taps.',
    'long_description': '# tap-workato\n\n`tap-workato` is a Singer tap for Workato.\n\nBuilt with the [Meltano Tap SDK](https://sdk.meltano.com) for Singer Taps.\n\n## Installation\nThis tap is installable via pypi\n\n```bash\npipx install tap-workato\n```\n\nOr, if using via Meltano you can add configuration like so in the `meltano.yml` file:\n\n```yaml\nplugins:\n  extractors:\n  - name: tap-workato\n    namespace: tap_workato\n    pip_url: -e .\n    capabilities:\n    - state\n    - catalog\n    - discover\n    settings:\n    - name: user_token\n      kind: password\n    - name: user_email\n      kind: password\n```\n\nand then run the following from the CLI:\n\n```shell\nmeltano install extractor tap-workato\n```\n\n## Configuration\n\n### Accepted Config Options\n\nA full list of supported settings and capabilities for this\ntap is available by running:\n\n```bash\ntap-workato --about\n```\n\nAvailable streams include:\n-  `api_clients`\n-  `api_collection`\n-  `connections`\n-  `customer_accounts` *\n-  `customer_api_access_profiles` *\n-  `customer_api_clients` *\n-  `customer_api_collections` *\n-  `customer_api_endpoints` *\n-  `customer_connections` *\n-  `customer_connections` *\n-  `customer_members` *\n-  `customer_recipes` *\n-  `customer_roles` *\n-  `folders`\n-  `jobs`\n-  `on_prem_agents`\n-  `on_prem_groups`\n-  `recipes`\n-  `roles`\n\nNotes:\n- _* The endpoints utilized by these streams are \n  Embedded Vendor APIs and require the `oem_vendor` privilege in Workato._\n- The workato Embedded API does not allow for extracting customer jobs \n  data at this time.\n\n### Source Authentication and Authorization\n\nYou will need authentication tokens set up in your Workato account. Namely a user \nemail and a user token. See the \ninstructions [here](https://docs.workato.com/oem/oem-api.html#authentication).\n\n## Usage\n\nYou can easily run `tap-workato` by itself or in a pipeline using [Meltano](https://meltano.com/).\n\n### Executing the Tap Directly\n\n```bash\ntap-workato --version\ntap-workato --help\ntap-workato --config CONFIG --discover > ./catalog.json\n```\n\n## Developer Resources\n\n### Initialize your Development Environment\n\n```bash\npipx install poetry\npoetry install\n```\n\n### Create and Run Tests\n\nCreate tests within the `tap_workato/tests` subfolder and\n  then run:\n\n```bash\npoetry run pytest\n```\n\nYou can also test the `tap-workato` CLI interface directly using `poetry run`:\n\n```bash\npoetry run tap-workato --help\n```\n\nOther useful workflows are included in the `tox.ini`:\n\n```bash\npoetry run tox -e format\npoetry run tox -e lint\npoetry run tox -e pytest\n```\n\n### Testing with [Meltano](https://www.meltano.com)\n\n_**Note:** This tap will work in any Singer environment and does not require Meltano.\nExamples here are for convenience and to streamline end-to-end orchestration scenarios._\n\nYour project comes with a custom `meltano.yml` project file already created. Open the `meltano.yml` and follow any _"TODO"_ items listed in\nthe file.\n\nNext, install Meltano (if you haven\'t already) and any needed plugins:\n\n```bash\n# Install meltano\npipx install meltano\n# Initialize meltano within this directory\ncd tap-workato\nmeltano install\n```\n\nNow you can test and orchestrate using Meltano:\n\n```bash\n# Test invocation:\nmeltano invoke tap-workato --version\n# OR run a test `elt` pipeline:\nmeltano elt tap-workato target-jsonl\n```\n\n### SDK Dev Guide\n\nSee the [dev guide](https://sdk.meltano.com/en/latest/dev_guide.html) for more instructions on how to use the SDK to \ndevelop your own taps and targets.\n',
    'author': 'Josh Lloyd',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Widen/tap-workato',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
